from collections import defaultdict
from collections.abc import Iterable, Sequence
from logging import getLogger

import google.protobuf.descriptor_pb2 as pb

import nebius.base.protos.pythonic_names as names
from nebius.api.nebius import (
    EnumPySDKSettings,
    EnumValuePySDKSettings,
    FieldPySDKSettings,
    MessagePySDKSettings,
    MethodPySDKSettings,
    OneofPySDKSettings,
    ServicePySDKSettings,
    enum_py_sdk,
    enum_value_py_sdk,
    field_py_sdk,
    message_py_sdk,
    method_py_sdk,
    oneof_py_sdk,
    service_py_sdk,
)
from nebius.base.protos.compiler.pygen import ImportedSymbol, ImportPath

log = getLogger(__name__)


class DescriptorError(Exception):
    pass


class FieldNotMessageError(DescriptorError):
    def __init__(self, field: "Field") -> None:
        super().__init__(
            f"Field {field.name} of {field.containing_message.full_type_name} is not "
            "a message type."
        )


class FieldNotEnumError(DescriptorError):
    def __init__(self, field: "Field") -> None:
        super().__init__(
            f"Field {field.name} of {field.containing_message.full_type_name} is not "
            "a enum type."
        )


class SourceInfo:
    def __init__(self, info: pb.SourceCodeInfo.Location | None = None) -> None:
        if info is not None:
            self._info = info
        else:
            self._info = pb.SourceCodeInfo.Location(
                leading_comments="",
                trailing_comments="",
                leading_detached_comments=[],
                span=[-1, -1, -1],
            )

    @property
    def leading_detached_comments(self) -> Sequence[str]:
        return self._info.leading_detached_comments

    @property
    def leading_comments(self) -> str:
        return self._info.leading_comments

    @property
    def trailing_comments(self) -> str:
        return self._info.trailing_comments

    @property
    def start_line(self) -> int:
        return self._info.span[0]

    @property
    def start_column(self) -> int:
        return self._info.span[1]

    @property
    def end_line(self) -> int:
        if len(self._info.span) > 3:
            return self._info.span[2]
        return self._info.span[0]

    @property
    def end_column(self) -> int:
        return self._info.span[-1]


class Descriptor:
    @property
    def name(self) -> str:
        return self.descriptor.name  # type: ignore

    @property
    def pythonic_name(self) -> str:
        raise NotImplementedError("Subclasses must implement pythonic_name property")


class DescriptorNameError(DescriptorError):
    def __init__(self, descriptor: Descriptor, error: Exception) -> None:
        super().__init__(f"{descriptor!r} has error with name: {error}")


class DuplicatesError(DescriptorError):
    def __init__(
        self,
        descriptor: Descriptor,
        duplicates: Iterable[list[Descriptor]],
        inner_errors: list["DuplicatesError"] | None = None,
    ) -> None:
        lines = "\n".join(
            f"{group[0].pythonic_name}: {group!r}" for group in duplicates
        )
        if lines != "":
            lines = f"{descriptor!r} has duplicate names for children:\n{lines}"
        if inner_errors is not None and len(inner_errors) > 0:
            if len(lines) != 0:
                lines += "\n"
            lines += "\n".join(str(e) for e in inner_errors)

        super().__init__(lines)


class EnumValue(Descriptor):
    def __init__(
        self,
        descriptor: pb.EnumValueDescriptorProto,
        containing_enum: "Enum",
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_enum = containing_enum
        self._pythonic_name = ""
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = EnumValuePySDKSettings(
            descriptor.options.Extensions[enum_value_py_sdk]  # type: ignore
        )

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self.containing_enum.containing_file.source_code_info:
            return SourceInfo(
                self.containing_enum.containing_file.source_code_info[self.path_in_file]
            )
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            self._path_in_file = tuple(
                tuple(self.containing_enum.path_in_file)
                + tuple(
                    [
                        pb.EnumDescriptorProto.VALUE_FIELD_NUMBER,
                        self._index,
                    ]
                )
            )
        return self._path_in_file

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.enum_value(
                    self.name,
                    self.containing_enum.name,
                    self._settings.name,
                    self.containing_enum.full_type_name + "." + self.name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def number(self) -> int:
        return self.descriptor.number  # type:ignore[no-any-return,unused-ignore]

    @property
    def pb2(self) -> ImportedSymbol:
        c_import = self.containing_enum.pb2
        return ImportedSymbol(c_import.name + "." + self.name, c_import.import_path)

    def __repr__(self) -> str:
        return f"EnumValue({self.containing_enum.full_type_name}.{self.name})"


class Enum(Descriptor):
    def __init__(
        self,
        descriptor: pb.EnumDescriptorProto,
        containing_file: "File",
        containing_message: "Message|None" = None,
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_message = containing_message
        self.containing_file = containing_file
        self._values: list[EnumValue] | None = None
        self._values_dict: dict[str, EnumValue] | None = None
        self._pythonic_name = ""
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = EnumPySDKSettings(
            descriptor.options.Extensions[enum_py_sdk]  # type: ignore
        )
        self._checked = False

    def check_names(self) -> None:
        if self._checked:
            return
        val_names = defaultdict[str, list[Descriptor]](list[Descriptor])
        for val in self.values:
            val_names[val.pythonic_name].append(val)

        duplicates: list[list[Descriptor]] = [
            group for group in val_names.values() if len(group) > 1
        ]
        if duplicates:
            raise DuplicatesError(self, duplicates)
        self._checked = True

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self.containing_file.source_code_info:
            return SourceInfo(self.containing_file.source_code_info[self.path_in_file])
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            if self.containing_message is not None:
                self._path_in_file = tuple(
                    tuple(self.containing_message.path_in_file)
                    + tuple(
                        [
                            pb.DescriptorProto.ENUM_TYPE_FIELD_NUMBER,
                            self._index,
                        ]
                    )
                )
            else:
                self._path_in_file = tuple(
                    [
                        pb.FileDescriptorProto.ENUM_TYPE_FIELD_NUMBER,
                        self._index,
                    ]
                )
        return self._path_in_file

    def __repr__(self) -> str:
        return f"Enum({self.full_type_name})"

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.enum(
                    self.full_type_name,
                    self._settings.name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def values(self) -> list[EnumValue]:
        if self._values is None:
            self._values = [
                EnumValue(val, self, index=i)
                for i, val in enumerate(self.descriptor.value)
            ]
        return self._values

    @property
    def values_dict(self) -> dict[str, EnumValue]:
        if self._values_dict is None:
            self._values_dict = {val.name: val for val in self.values}
        return self._values_dict

    @property
    def no_wrap(self) -> bool:
        if self.containing_file.skipped:
            return True
        return False

    @property
    def full_type_name(self) -> str:
        if self.containing_message is not None:
            return self.containing_message.full_type_name + "." + self.name
        return "." + self.containing_file.package + "." + self.name

    @property
    def export_path(self) -> ImportedSymbol:
        if self.containing_message is not None:
            c_import = self.containing_message.export_path
            return ImportedSymbol(
                c_import.name + "." + self.pythonic_name, c_import.import_path
            )
        return ImportedSymbol(self.pythonic_name, self.containing_file.export_path)

    @property
    def pb2(self) -> ImportedSymbol:
        if self.containing_message is not None:
            c_import = self.containing_message.pb2
            return ImportedSymbol(c_import.name + "." + self.name, c_import.import_path)
        return ImportedSymbol(self.name, self.containing_file.pb2)


class Field(Descriptor):
    def __init__(
        self,
        descriptor: pb.FieldDescriptorProto,
        containing_message: "Message",
        containing_file: "File",
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_message = containing_message
        self._pythonic_name = ""
        self._containing_file = containing_file
        self._oneof: "OneOf|None|bool" = None
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = FieldPySDKSettings(
            descriptor.options.Extensions[field_py_sdk]  # type: ignore
        )

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self._containing_file.source_code_info:
            return SourceInfo(self._containing_file.source_code_info[self.path_in_file])
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            self._path_in_file = tuple(
                tuple(self.containing_message.path_in_file)
                + tuple(
                    [
                        pb.DescriptorProto.FIELD_FIELD_NUMBER,
                        self._index,
                    ]
                )
            )
        return self._path_in_file

    def is_in_oneof(self) -> bool:
        return self.descriptor.HasField("oneof_index")

    @property
    def containing_oneof(self) -> "OneOf":
        if self._oneof is None:
            if not self.is_in_oneof():
                self._oneof = False
            else:
                self._oneof = self.containing_message.oneofs[
                    self.descriptor.oneof_index
                ]
        if isinstance(self._oneof, OneOf):
            return self._oneof
        raise ValueError("not in OneOf")

    @property
    def full_type_name(self) -> str:
        if self.is_extension():
            return "." + self._containing_file.package + "." + self.name
        return self.containing_message.full_type_name + "." + self.name

    def __repr__(self) -> str:
        return f"Field({self.full_type_name})"

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.field(
                    self.name,
                    self.containing_message.name,
                    self._settings.name,
                    self.full_type_name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def message(self) -> "Message":
        if self.descriptor.type == self.descriptor.TYPE_MESSAGE:
            return self.containing_message.get_message_by_type_name(
                self.descriptor.type_name
            )
        else:
            raise FieldNotMessageError(self)

    @property
    def enum(self) -> "Enum":
        if self.descriptor.type == self.descriptor.TYPE_ENUM:
            return self.containing_message.get_enum_by_type_name(
                self.descriptor.type_name
            )
        else:
            raise FieldNotEnumError(self)

    def tracks_presence(self) -> bool:
        return (  # type:ignore[no-any-return,unused-ignore]
            self.descriptor.proto3_optional
            or (
                self.descriptor.type == self.descriptor.TYPE_MESSAGE
                and self.descriptor.label != self.descriptor.LABEL_REPEATED
            )
            or self.descriptor.HasField("oneof_index")
        )

    def is_extension(self) -> bool:
        return self.descriptor.extendee != ""

    @property
    def pb2(self) -> ImportedSymbol:
        if self.is_extension():
            return ImportedSymbol(
                self.name,
                self._containing_file.pb2,
            )
        else:
            msg_pb2 = self.containing_message.pb2
            return ImportedSymbol(msg_pb2.name + "." + self.name, msg_pb2.import_path)

    @property
    def export_path(self) -> ImportedSymbol:
        if self.is_extension():
            return ImportedSymbol(
                self.pythonic_name,
                self._containing_file.export_path,
            )
        else:
            msg = self.containing_message.export_path
            return ImportedSymbol(msg.name + "." + self.pythonic_name, msg.import_path)

    @property
    def number(self) -> int:
        return self.descriptor.number

    @property
    def map_key(self) -> "Field":
        return self.message.field_by_name("key")

    @property
    def map_value(self) -> "Field":
        return self.message.field_by_name("value")

    def python_type(self) -> ImportedSymbol:
        match self.descriptor.type:
            case self.descriptor.TYPE_DOUBLE | self.descriptor.TYPE_FLOAT:
                return ImportedSymbol("float", "builtins")
            case (
                self.descriptor.TYPE_INT64
                | self.descriptor.TYPE_UINT64
                | self.descriptor.TYPE_INT32
                | self.descriptor.TYPE_UINT32
                | self.descriptor.TYPE_FIXED64
                | self.descriptor.TYPE_FIXED32
                | self.descriptor.TYPE_SFIXED32
                | self.descriptor.TYPE_SFIXED64
                | self.descriptor.TYPE_SINT32
                | self.descriptor.TYPE_SINT64
            ):
                return ImportedSymbol("int", "builtins")
            case self.descriptor.TYPE_BOOL:
                return ImportedSymbol("bool", "builtins")
            case self.descriptor.TYPE_STRING:
                return ImportedSymbol("str", "builtins")
            case self.descriptor.TYPE_BYTES:
                return ImportedSymbol("bytes", "builtins")
            case self.descriptor.TYPE_ENUM:
                return self.enum.export_path
            case self.descriptor.TYPE_MESSAGE:
                return self.message.export_path
            case _:
                raise ValueError(f"Unsupported descriptor type: {self.descriptor.type}")

    def is_enum(self) -> bool:
        if self.descriptor.type == self.descriptor.TYPE_ENUM:
            return True
        return False

    def is_map(self) -> bool:
        return (
            self.descriptor.label == self.descriptor.LABEL_REPEATED
            and self.descriptor.type == self.descriptor.TYPE_MESSAGE
            and self.message.is_map_entry()
        )

    def is_repeated(self) -> bool:
        return (
            self.descriptor.label == self.descriptor.LABEL_REPEATED
            and not self.is_map()
        )

    def is_message(self) -> bool:
        if self.descriptor.type == self.descriptor.TYPE_MESSAGE:
            return True
        return False


class OneOf(Descriptor):
    def __init__(
        self,
        descriptor: pb.OneofDescriptorProto,
        index: int,
        containing_message: "Message",
        containing_file: "File",
    ) -> None:
        self.descriptor = descriptor
        self.containing_message = containing_message
        self._index = index
        self._pythonic_name = ""
        self._containing_file = containing_file
        self._fields: "list[Field]|None" = None
        self._path_in_file: Sequence[int] | None = None
        self._settings = OneofPySDKSettings(
            descriptor.options.Extensions[oneof_py_sdk]  # type: ignore
        )

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self._containing_file.source_code_info:
            return SourceInfo(self._containing_file.source_code_info[self.path_in_file])
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            self._path_in_file = tuple(
                tuple(self.containing_message.path_in_file)
                + tuple(
                    [
                        pb.DescriptorProto.ONEOF_DECL_FIELD_NUMBER,
                        self._index,
                    ]
                )
            )
        return self._path_in_file

    @property
    def fields(self) -> list[Field]:
        if self._fields is None:
            self._fields = [
                f
                for f in self.containing_message.fields()
                if (
                    f.descriptor.HasField("oneof_index")
                    and f.descriptor.oneof_index == self._index
                )
            ]
        return self._fields

    @property
    def full_type_name(self) -> str:
        return self.containing_message.full_type_name + "." + self.name

    def __repr__(self) -> str:
        return f"OneOf({self.full_type_name})"

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.one_of(
                    self.name,
                    self.containing_message.name,
                    self._settings.name,
                    self.full_type_name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def export_path(self) -> ImportedSymbol:
        msg = self.containing_message.export_path
        return ImportedSymbol(msg.name + "." + self.pythonic_name, msg.import_path)


class Message(Descriptor):
    def __init__(
        self,
        descriptor: pb.DescriptorProto,
        containing_file: "File",
        containing_message: "Message|None" = None,
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_file = containing_file
        self.containing_message = containing_message
        self._messages: "list[Message]|None" = None
        self._messages_dict: "dict[str,Message]|None" = None
        self._fields: list[Field] | None = None
        self._fields_dict: dict[str, Field] | None = None
        self._enums_dict: dict[str, Enum] | None = None
        self._oneofs: list[OneOf] | None = None
        self._oneofs_dict: dict[str, OneOf] | None = None
        self.attached_names = dict[str, str]()
        self._pythonic_name = ""
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = MessagePySDKSettings(
            descriptor.options.Extensions[message_py_sdk]  # type: ignore
        )
        self._checked = False

    def check_names(self) -> None:
        if self._checked:
            return
        val_names = defaultdict[str, list[Descriptor]](list[Descriptor])
        inner_errors: list[DuplicatesError] = []
        for field in self.fields():
            val_names[field.pythonic_name].append(field)
        for msg in self.messages():
            val_names[msg.pythonic_name].append(msg)
            try:
                msg.check_names()
            except DuplicatesError as e:
                inner_errors.append(e)
        for enum in self.enums:
            val_names[enum.pythonic_name].append(enum)
            try:
                enum.check_names()
            except DuplicatesError as e:
                inner_errors.append(e)
        for oneof in self.oneofs:
            val_names[oneof.pythonic_name].append(oneof)
        duplicates: list[list[Descriptor]] = [
            v for v in val_names.values() if len(v) > 1
        ]
        if duplicates or len(inner_errors) > 0:
            raise DuplicatesError(self, duplicates, inner_errors)
        self._checked = True

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self.containing_file.source_code_info:
            return SourceInfo(self.containing_file.source_code_info[self.path_in_file])
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            if self.containing_message is not None:
                self._path_in_file = tuple(
                    tuple(self.containing_message.path_in_file)
                    + tuple(
                        [
                            pb.DescriptorProto.NESTED_TYPE_FIELD_NUMBER,
                            self._index,
                        ]
                    )
                )
            else:
                self._path_in_file = tuple(
                    [
                        pb.FileDescriptorProto.MESSAGE_TYPE_FIELD_NUMBER,
                        self._index,
                    ]
                )
        return self._path_in_file

    def __repr__(self) -> str:
        return f"Message({self.full_type_name})"

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.message(
                    self.full_type_name,
                    self._settings.name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def enums_dict(self) -> dict[str, Enum]:
        if self._enums_dict is None:
            self._enums_dict = {
                e.name: Enum(e, self.containing_file, self, index=i)
                for i, e in enumerate(self.descriptor.enum_type)
            }
        return self._enums_dict

    @property
    def oneofs(self) -> list[OneOf]:
        if self._oneofs is None:
            self._oneofs = [
                OneOf(o, i, self, self.containing_file)
                for i, o in enumerate(self.descriptor.oneof_decl)
            ]
        return self._oneofs

    @property
    def oneofs_dict(self) -> dict[str, OneOf]:
        if self._oneofs_dict is None:
            self._oneofs_dict = {o.name: o for o in self.oneofs}
        return self._oneofs_dict

    @property
    def enums(self) -> Iterable[Enum]:
        return self.enums_dict.values()

    @property
    def fields_dict(self) -> dict[str, Field]:
        if self._fields_dict is None:
            self._fields_dict = {field.name: field for field in self.fields()}
        return self._fields_dict

    @property
    def no_wrap(self) -> bool:
        if self.containing_file.skipped:
            return True
        return False

    @property
    def export_path(self) -> ImportedSymbol:
        if self.containing_message is not None:
            c_import = self.containing_message.export_path
            return ImportedSymbol(
                c_import.name + "." + self.pythonic_name, c_import.import_path
            )
        return ImportedSymbol(self.pythonic_name, self.containing_file.export_path)

    def field_by_name(self, name: str) -> Field:
        return self.fields_dict[name]

    def fields(self) -> list[Field]:
        if self._fields is None:
            self._fields = [
                Field(f, self, self.containing_file, index=i)
                for i, f in enumerate(self.descriptor.field)
            ]
        return self._fields

    @property
    def full_type_name(self) -> str:
        if self.containing_message is not None:
            return self.containing_message.full_type_name + "." + self.name
        return "." + self.containing_file.package + "." + self.name

    def get_message_by_type_name(self, name: str, strict: bool = False) -> "Message":
        if name[0] == ".":
            return self.containing_file.get_message_by_type_name(name)
        name_parts = name.split(".", 1)
        try:
            msg = self.message_by_name(name_parts[0])
            if len(name_parts) > 1:
                return msg.get_message_by_type_name(name_parts[1], strict=True)
            return msg
        except KeyError:
            if strict:
                raise KeyError(
                    f"Message {name} not found in scope of " f"{self.full_type_name}"
                )
            if self.containing_message is not None:
                return self.containing_message.get_message_by_type_name(name)
            else:
                return self.containing_file.get_message_by_type_name(name)

    def get_enum_by_type_name(self, name: str, strict: bool = False) -> "Enum":
        if name[0] == ".":
            return self.containing_file.get_enum_by_type_name(name)
        name_parts = name.split(".", 1)
        try:
            if len(name_parts) > 1:
                msg = self.message_by_name(name_parts[0])
                return msg.get_enum_by_type_name(name_parts[1], strict=True)
            return self.enums_dict[name_parts[0]]
        except KeyError:
            if strict:
                raise KeyError(
                    f"Enum {name} not found in scope of " f"{self.full_type_name}"
                )
            if self.containing_message is not None:
                return self.containing_message.get_enum_by_type_name(name)
            else:
                return self.containing_file.get_enum_by_type_name(name)

    def message_by_name(self, name: str) -> "Message":
        if self._messages_dict is None:
            self._messages_dict = {msg.name: msg for msg in self.messages()}
        return self._messages_dict[name]

    def messages(self) -> Sequence["Message"]:
        if self._messages is None:
            self._messages = [
                Message(msg, self.containing_file, self, index=i)
                for i, msg in enumerate(self.descriptor.nested_type)
            ]
        return self._messages

    def is_map_entry(self) -> bool:
        if self.descriptor.options.map_entry:
            return True
        return False

    @property
    def pb2(self) -> ImportedSymbol:
        if self.containing_message is not None:
            c_import = self.containing_message.pb2
            return ImportedSymbol(c_import.name + "." + self.name, c_import.import_path)
        return ImportedSymbol(self.name, self.containing_file.pb2)

    def collect_all_names(self) -> set[str]:
        ret = set[str]([field.pythonic_name for field in self.fields()])
        for msg in self.messages():
            ret.add(msg.pythonic_name)
            ret = ret.union(msg.collect_all_names())
        for enum in self.enums:
            ret.add(enum.pythonic_name)
            ret = ret.union([v.pythonic_name for v in enum.values])
        for oneof in self.oneofs:
            ret.add(oneof.pythonic_name)
        return ret


class Method(Descriptor):
    def __init__(
        self,
        descriptor: pb.MethodDescriptorProto,
        containing_service: "Service",
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_service = containing_service
        self._pythonic_name = ""
        self._input: Message | None = None
        self._output: Message | None = None
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = MethodPySDKSettings(
            self.descriptor.options.Extensions[method_py_sdk]  # type: ignore
        )

    @property
    def source_info(self) -> SourceInfo:
        if (
            self.path_in_file
            in self.containing_service.containing_file.source_code_info
        ):
            return SourceInfo(
                self.containing_service.containing_file.source_code_info[
                    self.path_in_file
                ]
            )
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            self._path_in_file = tuple(
                tuple(self.containing_service.path_in_file)
                + tuple(
                    [
                        pb.ServiceDescriptorProto.METHOD_FIELD_NUMBER,
                        self._index,
                    ]
                )
            )
        return self._path_in_file

    def __repr__(self) -> str:
        return f"Method({self.full_type_name})"

    @property
    def input(self) -> Message:
        if self._input is None:
            self._input = (
                self.containing_service.containing_file.get_message_by_type_name(
                    self.descriptor.input_type
                )
            )
        return self._input

    @property
    def output(self) -> Message:
        if self._output is None:
            self._output = (
                self.containing_service.containing_file.get_message_by_type_name(
                    self.descriptor.output_type
                )
            )
        return self._output

    @property
    def full_type_name(self) -> str:
        return self.containing_service.full_type_name + "." + self.name

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.method(
                    self.name,
                    self.containing_service.pythonic_name,
                    self._settings.name,
                    self.full_type_name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name


class Service(Descriptor):
    def __init__(
        self,
        descriptor: pb.ServiceDescriptorProto,
        containing_file: "File",
        index: int = -1,
    ) -> None:
        self.descriptor = descriptor
        self.containing_file = containing_file
        self._pythonic_name = ""
        self._methods: dict[str, Method] | None = None
        self._index = index
        self._path_in_file: Sequence[int] | None = None
        self._settings = ServicePySDKSettings(
            self.descriptor.options.Extensions[service_py_sdk]  # type: ignore
        )
        self._checked = False

    def check_names(self) -> None:
        if self._checked:
            return

        val_names = defaultdict[str, list[Descriptor]](list[Descriptor])
        for method in self.methods.values():
            val_names[method.pythonic_name].append(method)
        duplicates: list[list[Descriptor]] = [
            v for v in val_names.values() if len(v) > 1
        ]
        if duplicates:
            raise DuplicatesError(self, duplicates)
        self._checked = True

    @property
    def source_info(self) -> SourceInfo:
        if self.path_in_file in self.containing_file.source_code_info:
            return SourceInfo(self.containing_file.source_code_info[self.path_in_file])
        return SourceInfo()

    @property
    def path_in_file(self) -> Sequence[int]:
        if self._path_in_file is None:
            self._path_in_file = tuple(
                [
                    pb.FileDescriptorProto.SERVICE_FIELD_NUMBER,
                    self._index,
                ]
            )
        return self._path_in_file

    def __repr__(self) -> str:
        return f"Service({self.full_type_name})"

    @property
    def methods(self) -> dict[str, Method]:
        if self._methods is None:
            self._methods = {
                m.name: Method(m, self, index=i)
                for i, m in enumerate(self.descriptor.method)
            }
        return self._methods

    @property
    def export_path(self) -> ImportedSymbol:
        return ImportedSymbol(self.pythonic_name, self.containing_file.export_path)

    @property
    def pythonic_name(self) -> str:
        if self._pythonic_name == "":
            try:
                self._pythonic_name = names.service(
                    self.full_type_name,
                    self._settings.name,
                )
            except names.NameError as e:
                raise DescriptorNameError(self, e) from None
        return self._pythonic_name

    @property
    def full_type_name(self) -> str:
        return "." + self.containing_file.package + "." + self.name

    def collect_all_names(self) -> set[str]:
        ret = set[str]([m.pythonic_name for m in self.methods.values()])
        return ret


class File(Descriptor):
    def __init__(
        self,
        descriptor: pb.FileDescriptorProto,
        file_set: "FileSet",
        is_generated: bool,
    ) -> None:
        self.descriptor = descriptor
        self.global_set = file_set
        self.is_generated = is_generated
        import_path = (
            self.descriptor.name.removesuffix(".proto").replace("/", ".") + "_pb2"
        )
        for prefix, subst in file_set.import_substitutions.items():
            if import_path.startswith(prefix + ".") or import_path == prefix:
                import_path = subst + import_path.removeprefix(prefix)
                break
        self.pb2: ImportPath = ImportPath(import_path)
        self.skipped: bool = self.global_set.is_package_skipped(self.package)
        if self.skipped:
            self.export_path: ImportPath = self.pb2
        else:
            export_path = self.descriptor.package
            export_substituted = False
            for prefix, subst in file_set.export_substitutions.items():
                if export_path.startswith(prefix + ".") or export_path == prefix:
                    export_path = subst + export_path.removeprefix(prefix)
                    export_substituted = True
                    break
            if not export_substituted:
                for prefix, subst in file_set.import_substitutions.items():
                    if export_path.startswith(prefix + ".") or export_path == prefix:
                        export_path = subst + export_path.removeprefix(prefix)
                        break
            self.export_path = ImportPath(export_path)
        self._messages: list[Message] | None = None
        self._messages_dict: dict[str, Message] | None = None
        self._deps_dict: dict[str, File] | None = None
        self._enums_dict: dict[str, Enum] | None = None
        self._services_dict: dict[str, Service] | None = None
        self._extensions: dict[str, Field] | None = None
        self._source_code_info: (
            dict[Sequence[int], pb.SourceCodeInfo.Location] | None
        ) = None
        self._checked: defaultdict[str, list[Descriptor]] | None = None

    def check_names(self) -> defaultdict[str, list[Descriptor]]:
        if self._checked is not None:
            return self._checked
        val_names = defaultdict[str, list[Descriptor]](list[Descriptor])
        inner_errors = list[DuplicatesError]()
        for srv in self.services_dict.values():
            val_names[srv.pythonic_name].append(srv)
            try:
                srv.check_names()
            except DuplicatesError as e:
                inner_errors.append(e)
        for msg in self.messages():
            val_names[msg.pythonic_name].append(msg)
            try:
                msg.check_names()
            except DuplicatesError as e:
                inner_errors.append(e)
        for enum in self.enums:
            val_names[enum.pythonic_name].append(enum)
            try:
                enum.check_names()
            except DuplicatesError as e:
                inner_errors.append(e)
        for ext in self.extensions.values():
            val_names[ext.pythonic_name].append(ext)

        duplicates: list[list[Descriptor]] = [
            v for v in val_names.values() if len(v) > 1
        ]
        if duplicates or len(inner_errors) > 0:
            raise DuplicatesError(self, duplicates, inner_errors)
        self._checked = val_names
        return self._checked

    @property
    def source_code_info(self) -> dict[Sequence[int], pb.SourceCodeInfo.Location]:
        if self._source_code_info is None:
            self._source_code_info = {
                tuple(loc.path): loc
                for loc in self.descriptor.source_code_info.location
            }
        return self._source_code_info

    def __repr__(self) -> str:
        return f"File({self.name})"

    @property
    def extensions(self) -> dict[str, Field]:
        if self._extensions is None:
            self._extensions = {
                x.name: Field(
                    x,
                    self.get_message_by_type_name(x.extendee),
                    self,
                )
                for x in self.descriptor.extension
            }
        return self._extensions

    def collect_all_names(self, with_locals: bool = True) -> set[str]:
        ret = set[str](self.package.split("."))
        for msg in self.messages():
            ret.add(msg.pythonic_name)
            if with_locals:
                ret = ret.union(msg.collect_all_names())
        for enum in self.enums:
            ret.add(enum.pythonic_name)
            ret = ret.union([v.pythonic_name for v in enum.values])
        for srv in self.services_dict.values():
            ret.add(srv.pythonic_name)
            ret = ret.union(srv.collect_all_names())
        for ext in self.extensions.values():
            ret.add(ext.pythonic_name)
        if with_locals:
            for dep in self.dependencies.values():
                ret = ret.union(dep.collect_all_names(True))
        return ret

    def get_message_by_type_name(self, name: str, strict: bool = False) -> "Message":
        name_partial = name
        if name_partial[0] == ".":
            if name_partial.startswith("." + self.package + "."):
                strict = True
                name_partial = name_partial.removeprefix("." + self.package + ".")
        name_parts = name_partial.split(".", 1)
        try:
            msg = self.message_by_name(name_parts[0])
            if len(name_parts) > 1:
                return msg.get_message_by_type_name(name_parts[1], strict=True)
            return msg
        except KeyError:
            for dep in self.dependencies.values():
                if (
                    strict
                    and dep.package != self.package
                    and not dep.package.startswith(self.package + ".")
                ):
                    continue
                try:
                    return dep.get_message_by_type_name(name)
                except KeyError:
                    pass
            raise KeyError(f"Message {name} not found in scope of {self.name}")

    def get_enum_by_type_name(self, name: str, strict: bool = False) -> "Enum":
        name_partial = name
        if name_partial[0] == ".":
            if name_partial.startswith("." + self.package + "."):
                strict = True
                name_partial = name_partial.removeprefix("." + self.package + ".")
        name_parts = name_partial.split(".", 1)
        try:
            if len(name_parts) > 1:
                msg = self.message_by_name(name_parts[0])
                return msg.get_enum_by_type_name(name_parts[1], strict=True)
            return self.enums_dict[name_parts[0]]
        except KeyError:
            for dep in self.dependencies.values():
                if (
                    strict
                    and dep.package != self.package
                    and not dep.package.startswith(self.package + ".")
                ):
                    continue
                try:
                    return dep.get_enum_by_type_name(name)
                except KeyError:
                    pass
            raise KeyError(f"Enum {name} not found in scope of {self.name}")

    @property
    def dependencies(self) -> "dict[str, File]":
        if self._deps_dict is None:
            self._deps_dict = {
                name: self.global_set.file_by_name(name)
                for name in self.descriptor.dependency
            }
        return self._deps_dict

    @property
    def package(self) -> str:
        return str(self.descriptor.package)

    def get_field_by_type_name(self, full_field_name: str) -> Field:
        msg_name, field_name = full_field_name.rsplit(".", 2)
        msg = self.get_message_by_type_name(msg_name)
        return msg.field_by_name(field_name)

    def get_extension_by_type_name(self, name: str, strict: bool = False) -> Field:
        name_partial = name
        if name_partial[0] == ".":
            if name_partial.startswith("." + self.package + "."):
                strict = True
                name_partial = name_partial.removeprefix("." + self.package + ".")
        try:
            return self.extensions[name_partial]
        except KeyError:
            for dep in self.dependencies.values():
                if (
                    strict
                    and dep.package != self.package
                    and not dep.package.startswith(self.package + ".")
                ):
                    continue
                try:
                    return dep.get_extension_by_type_name(name)
                except KeyError:
                    pass
            raise KeyError(f"Extension {name} not found in scope of {self.name}")

    def message_by_name(self, name: str) -> Message:
        if self._messages_dict is None:
            self._messages_dict = {msg.name: msg for msg in self.messages()}
        return self._messages_dict[name]

    def messages(self) -> Sequence[Message]:
        if self._messages is None:
            self._messages = [
                Message(msg, self, index=i)
                for i, msg in enumerate(self.descriptor.message_type)
            ]
        return self._messages

    @property
    def services_dict(self) -> dict[str, Service]:
        if self._services_dict is None:
            self._services_dict = {
                s.name: Service(s, self, index=i)
                for i, s in enumerate(self.descriptor.service)
            }
        return self._services_dict

    @property
    def enums_dict(self) -> dict[str, Enum]:
        if self._enums_dict is None:
            self._enums_dict = {
                e.name: Enum(e, self, index=i)
                for i, e in enumerate(self.descriptor.enum_type)
            }
        return self._enums_dict

    @property
    def enums(self) -> Iterable[Enum]:
        return self.enums_dict.values()

    @property
    def grpc(self) -> ImportPath:
        p = self.pb2
        return ImportPath(p.import_path + "_grpc")


class FileSet(Descriptor):
    def __init__(
        self,
        file_set: Sequence[pb.FileDescriptorProto],
        files_to_generate: Iterable[str],
        import_substitutions: dict[str, str] | None = None,
        export_substitutions: dict[str, str] | None = None,
        skip_packages: list[str] | None = None,
    ):
        if import_substitutions is None:
            import_substitutions = dict[str, str]()
        if export_substitutions is None:
            export_substitutions = dict[str, str]()
        if skip_packages is None:
            skip_packages = list[str]()

        self.import_substitutions = import_substitutions
        self.export_substitutions = export_substitutions
        self.skip_packages = set(skip_packages)
        self.files_to_generate = frozenset(files_to_generate)

        self._file_set = [
            File(file, self, file.name in self.files_to_generate) for file in file_set
        ]
        self._files_dict: dict[str, File] | None = None
        self._checked = False

    def check_names(self) -> None:
        if self._checked:
            return

        val_names = defaultdict[str, defaultdict[str, list[Descriptor]]](
            lambda: defaultdict(list[Descriptor])
        )
        inner_errors: list[DuplicatesError] = []
        for file in self.files_generated:
            try:
                file_children = file.check_names()
                for name, child in file_children.items():
                    val_names[file.package][name].extend(child)
            except DuplicatesError as e:
                inner_errors.append(e)
        for pkg_names in val_names.values():
            duplicates = [v for v in pkg_names.values() if len(v) > 1]
            if duplicates:
                inner_errors.append(
                    DuplicatesError(
                        self,
                        duplicates,
                    )
                )
        if inner_errors:
            raise DuplicatesError(self, [], inner_errors)
        self._checked = True

    def is_package_skipped(self, package: str) -> bool:
        for pkg in self.skip_packages:
            if package == pkg or package.startswith(pkg + "."):
                return True
        return False

    def collect_names(self, package: str) -> set[str]:
        ret = set[str](package.split("."))
        for file in self.files:
            if file.package == package:
                ret = ret.union(file.collect_all_names())
        return ret

    def file_by_name(self, name: str) -> "File":
        return self.files_dict[name]

    @property
    def files_dict(self) -> dict[str, File]:
        if self._files_dict is None:
            self._files_dict = {file.name: file for file in self._file_set}
        return self._files_dict

    @property
    def files(self) -> Sequence[File]:
        return self._file_set

    @property
    def files_generated(self) -> Sequence[File]:
        return [f for f in self.files if f.is_generated]



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ResourceMetadata(_message.Message):
    __slots__ = ["id", "parent_id", "name", "resource_version", "created_at", "updated_at", "labels"]
    class LabelsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_VERSION_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: _ClassVar[int]
    LABELS_FIELD_NUMBER: _ClassVar[int]
    id: str
    parent_id: str
    name: str
    resource_version: int
    created_at: _timestamp_pb2.Timestamp
    updated_at: _timestamp_pb2.Timestamp
    labels: _containers.ScalarMap[str, str]
    def __init__(self, id: _Optional[str] = ..., parent_id: _Optional[str] = ..., name: _Optional[str] = ..., resource_version: _Optional[int] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updated_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., labels: _Optional[_Mapping[str, str]] = ...) -> None: ...

class GetByNameRequest(_message.Message):
    __slots__ = ["parent_id", "name"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    name: str
    def __init__(self, parent_id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...



"""Static token bearer and receiver implementations.

This module contains simple bearer and receiver implementations useful for
local testing or when a fixed access token is made available (for example via
an environment variable). The ``EnvBearer`` reads the token from an
environment variable (by default :data:`nebius.base.constants.TOKEN_ENV`) and
``Bearer`` accepts a pre-configured :class:`Token` instance or raw token
string.

Examples
--------
Create a bearer from an explicit token string::

    from nebius.aio.token.static import Bearer

    bearer = Bearer("my-static-token")
    receiver = bearer.receiver()
    token = await receiver.fetch()

Use an environment variable::

    import os
    os.environ["NEBIUS_IAM_TOKEN"] = "token-from-env"
    from nebius.aio.token.static import EnvBearer

    bearer = EnvBearer()  # reads NEBIUS_IAM_TOKEN by default
    receiver = bearer.receiver()
    token = await receiver.fetch()
"""

import os
from logging import getLogger

from nebius.base.constants import TOKEN_ENV
from nebius.base.error import SDKError

from .token import Bearer as ParentBearer
from .token import Receiver as ParentReceiver
from .token import Token

log = getLogger(__name__)


class NoTokenInEnvError(SDKError):
    """Raised when an expected environment variable does not contain a token."""

    def __init__(self, env: str) -> None:
        """Initialize the error.

        :param env: Environment variable name that was expected to contain a
            token but was empty or missing.
        """
        super().__init__(f"No token found in the environment variable: {env}")


class Receiver(ParentReceiver):
    """A receiver that returns a pre-configured static token.

    This receiver is useful when the token is known in advance (for
    example embedded in configuration or provided via an environment
    variable). The receiver simply returns the supplied :class:`Token` and
    never indicates that a retry would be useful.

    :param token: Token instance to be returned by :meth:`_fetch`.
    """

    def __init__(self, token: Token) -> None:
        """Create a static receiver using the token."""
        super().__init__()
        self._latest = token

    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        """Return the configured token.

        The receiver returns the pre-configured token. If no token is configured
        `SDKError` is raised since the class invariant is that a static receiver must
        be constructed with a token instance.

        :param timeout: Ignored for static receiver but accepted for API
            compatibility.
        :param options: Ignored. Present for API compatibility.
        :returns: The configured :class:`Token`.
        :raises SDKError: If the internal token is `None` (should not happen when used
            correctly).
        """
        if self._latest is None:
            raise SDKError("Token has to be set")
        log.debug("static token fetched")
        return self._latest

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Static tokens are immutable and cannot be refreshed; never retry.

        :param err: The exception that caused a previous authentication
            failure (ignored).
        :param options: Ignored; present for API compatibility.
        :returns: Always `False`.
        """
        return False


class Bearer(ParentBearer):
    """Bearer that supplies a static token to authenticators.

    The bearer may be constructed with either a :class:`Token` instance or a
    raw token string. When a string is provided it is converted into a
    :class:`Token`. Empty token strings are rejected.

    :param token: :class:`Token` instance or raw token string.
    :type token: :class:`Token` or `str`
    :raises SDKError: When an empty token string is provided.

    Example
    -------

    Construct a bearer and use it to initialize the SDK::

        from nebius.sdk import SDK
        from nebius.aio.token.static import Bearer

        sdk = SDK(credentials=Bearer("my-static-token"))
    """

    def __init__(self, token: Token | str) -> None:
        """Initialize the static token bearer."""
        if isinstance(token, str):
            token = Token(token)
        if token.token == "":
            raise SDKError("empty token provided")
        super().__init__()
        self._tok = token

    def receiver(self) -> Receiver:
        """Return a :class:`Receiver` that yields the configured token.

        :returns: A :class:`Receiver` bound to the static token.
        """
        return Receiver(self._tok)


class EnvBearer(Bearer):
    """Bearer that reads the token from an environment variable.

    :param env_var_name: Environment variable name to read the token from.
        Defaults to :data:`nebius.base.constants.TOKEN_ENV`.
    :type env_var_name: `str`
    :raises NoTokenInEnvError: When the environment variable is not set or
        empty.

    Example
    -------

    Construct a bearer and use it to initialize the SDK::

        import os
        from nebius.sdk import SDK
        from nebius.aio.token.static import EnvBearer

        os.environ["NEBIUS_IAM_TOKEN"] = "token-from-env"
        sdk = SDK(credentials=EnvBearer())
    """

    def __init__(self, env_var_name: str = TOKEN_ENV) -> None:
        """Initialize the environment-variable based bearer."""
        val = os.environ.get(env_var_name, "")
        if val == "":
            raise NoTokenInEnvError(env_var_name)
        super().__init__(val)



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.error.v1alpha1 import common_errors_pb2 as _common_errors_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ServiceError(_message.Message):
    __slots__ = ["service", "code", "bad_request", "bad_resource_state", "resource_not_found", "resource_already_exists", "out_of_range", "permission_denied", "resource_conflict", "operation_aborted", "too_many_requests", "quota_failure", "internal_error", "retry_type"]
    class RetryType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[ServiceError.RetryType]
        CALL: _ClassVar[ServiceError.RetryType]
        UNIT_OF_WORK: _ClassVar[ServiceError.RetryType]
        NOTHING: _ClassVar[ServiceError.RetryType]
    UNSPECIFIED: ServiceError.RetryType
    CALL: ServiceError.RetryType
    UNIT_OF_WORK: ServiceError.RetryType
    NOTHING: ServiceError.RetryType
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    BAD_REQUEST_FIELD_NUMBER: _ClassVar[int]
    BAD_RESOURCE_STATE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_NOT_FOUND_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ALREADY_EXISTS_FIELD_NUMBER: _ClassVar[int]
    OUT_OF_RANGE_FIELD_NUMBER: _ClassVar[int]
    PERMISSION_DENIED_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_CONFLICT_FIELD_NUMBER: _ClassVar[int]
    OPERATION_ABORTED_FIELD_NUMBER: _ClassVar[int]
    TOO_MANY_REQUESTS_FIELD_NUMBER: _ClassVar[int]
    QUOTA_FAILURE_FIELD_NUMBER: _ClassVar[int]
    INTERNAL_ERROR_FIELD_NUMBER: _ClassVar[int]
    RETRY_TYPE_FIELD_NUMBER: _ClassVar[int]
    service: str
    code: str
    bad_request: _common_errors_pb2.BadRequest
    bad_resource_state: _common_errors_pb2.BadResourceState
    resource_not_found: _common_errors_pb2.ResourceNotFound
    resource_already_exists: _common_errors_pb2.ResourceAlreadyExists
    out_of_range: _common_errors_pb2.OutOfRange
    permission_denied: _common_errors_pb2.PermissionDenied
    resource_conflict: _common_errors_pb2.ResourceConflict
    operation_aborted: _common_errors_pb2.OperationAborted
    too_many_requests: _common_errors_pb2.TooManyRequests
    quota_failure: _common_errors_pb2.QuotaFailure
    internal_error: _common_errors_pb2.InternalError
    retry_type: ServiceError.RetryType
    def __init__(self, service: _Optional[str] = ..., code: _Optional[str] = ..., bad_request: _Optional[_Union[_common_errors_pb2.BadRequest, _Mapping]] = ..., bad_resource_state: _Optional[_Union[_common_errors_pb2.BadResourceState, _Mapping]] = ..., resource_not_found: _Optional[_Union[_common_errors_pb2.ResourceNotFound, _Mapping]] = ..., resource_already_exists: _Optional[_Union[_common_errors_pb2.ResourceAlreadyExists, _Mapping]] = ..., out_of_range: _Optional[_Union[_common_errors_pb2.OutOfRange, _Mapping]] = ..., permission_denied: _Optional[_Union[_common_errors_pb2.PermissionDenied, _Mapping]] = ..., resource_conflict: _Optional[_Union[_common_errors_pb2.ResourceConflict, _Mapping]] = ..., operation_aborted: _Optional[_Union[_common_errors_pb2.OperationAborted, _Mapping]] = ..., too_many_requests: _Optional[_Union[_common_errors_pb2.TooManyRequests, _Mapping]] = ..., quota_failure: _Optional[_Union[_common_errors_pb2.QuotaFailure, _Mapping]] = ..., internal_error: _Optional[_Union[_common_errors_pb2.InternalError, _Mapping]] = ..., retry_type: _Optional[_Union[ServiceError.RetryType, str]] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.iam.v1 import state_pb2 as _state_pb2
from nebius.api.nebius.iam.v1 import suspension_state_pb2 as _suspension_state_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Container(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: ContainerSpec
    status: ContainerStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[ContainerSpec, _Mapping]] = ..., status: _Optional[_Union[ContainerStatus, _Mapping]] = ...) -> None: ...

class ContainerSpec(_message.Message):
    __slots__ = ["region"]
    REGION_FIELD_NUMBER: _ClassVar[int]
    region: str
    def __init__(self, region: _Optional[str] = ...) -> None: ...

class ContainerStatus(_message.Message):
    __slots__ = ["suspension_state", "container_state", "region"]
    SUSPENSION_STATE_FIELD_NUMBER: _ClassVar[int]
    CONTAINER_STATE_FIELD_NUMBER: _ClassVar[int]
    REGION_FIELD_NUMBER: _ClassVar[int]
    suspension_state: _suspension_state_pb2.SuspensionState
    container_state: _state_pb2.State
    region: str
    def __init__(self, suspension_state: _Optional[_Union[_suspension_state_pb2.SuspensionState, str]] = ..., container_state: _Optional[_Union[_state_pb2.State, str]] = ..., region: _Optional[str] = ...) -> None: ...



import os
import subprocess
import sys
import tempfile

# Define the path to the mover script
MOVER_SCRIPT_PATH = "./src/nebius/base/protos/compiler/mover.py"


def run_mover(
    input_code: str, prefix_args: list[str]
) -> tuple[subprocess.CompletedProcess[str], str]:
    """
    Helper function to run the mover script with given input code and prefix arguments
    """
    with (
        tempfile.NamedTemporaryFile(delete=False, suffix=".py") as input_file,
        tempfile.NamedTemporaryFile(delete=False, suffix=".py") as output_file,
    ):
        input_file.write(input_code.encode("utf-8"))
        input_file.flush()

        # Run the mover script as a subprocess
        result = subprocess.run(
            [
                sys.executable,
                MOVER_SCRIPT_PATH,
                "--input",
                input_file.name,
                "--output",
                output_file.name,
                "--prefix",
            ]
            + prefix_args,
            capture_output=True,
            text=True,
        )

        # Read output
        with open(output_file.name, "r") as f:
            output_code = f.read()

    # Cleanup temporary files
    os.remove(input_file.name)
    os.remove(output_file.name)

    return result, output_code


def test_simple_import() -> None:
    """Test simple import transformation"""
    input_code = "import A.foo.bar"
    expected_output = "import B.foo.bar"

    result, output_code = run_mover(input_code, ["A=B"])

    assert result.returncode == 0
    assert output_code == expected_output


def test_inner_import() -> None:
    """Test inner import transformation"""
    input_code = "import A.foo.bar"
    expected_output = "import B.baz.bar"

    result, output_code = run_mover(input_code, ["A.foo=B.baz"])

    assert result.returncode == 0
    assert output_code == expected_output


def test_from_import() -> None:
    """Test 'from ... import ...' transformation"""
    input_code = "from A.foo import bar"
    expected_output = "from B.foo import bar"

    result, output_code = run_mover(input_code, ["A=B"])

    assert result.returncode == 0
    assert output_code == expected_output


def test_multiple_prefixes() -> None:
    """Test with multiple prefixes"""
    input_code = """
import A.foo # some comment

import C.baz
import abc.de.baz
import abe.de.baz
import ghi

# no changes
import aghi
import a.ghi
import a.ghi.foo

"""
    expected_output = """
import B.foo # some comment

import D.baz
import foo.bar.de.baz
import foe.baz
import foo.ghi

# no changes
import aghi
import a.ghi
import a.ghi.foo

"""

    result, output_code = run_mover(
        input_code, ["A=B", "C=D", "abc=foo.bar", "abe.de=foe", "ghi=foo.ghi"]
    )

    assert result.returncode == 0
    assert output_code == expected_output


def test_invalid_prefix_format() -> None:
    """Test invalid prefix format raises error"""
    result, _ = run_mover("import A.foo", ["AB"])

    assert result.returncode != 0
    assert "Invalid prefix format" in result.stderr


def test_no_transformation_needed() -> None:
    """Test that code remains the same if no matching prefixes"""
    input_code = "import X.y.z"
    expected_output = "import X.y.z"

    result, output_code = run_mover(input_code, ["A=B"])

    assert result.returncode == 0
    assert output_code == expected_output



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/audit/v2/resource_state.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$nebius/audit/v2/resource_state.proto\x12\x0fnebius.audit.v2\x1a\x1cgoogle/protobuf/struct.proto\"w\n\rResourceState\x12\x33\n\x08previous\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructR\x08previous\x12\x31\n\x07\x63urrent\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructR\x07\x63urrentB]\n\x16\x61i.nebius.pub.audit.v2B\x12ResourceStateProtoP\x01Z-github.com/nebius/gosdk/proto/nebius/audit/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.audit.v2.resource_state_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ai.nebius.pub.audit.v2B\022ResourceStateProtoP\001Z-github.com/nebius/gosdk/proto/nebius/audit/v2'
  _globals['_RESOURCESTATE']._serialized_start=87
  _globals['_RESOURCESTATE']._serialized_end=206
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Route(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: RouteSpec
    status: RouteStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[RouteSpec, _Mapping]] = ..., status: _Optional[_Union[RouteStatus, _Mapping]] = ...) -> None: ...

class RouteSpec(_message.Message):
    __slots__ = ["description", "destination", "next_hop"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    DESTINATION_FIELD_NUMBER: _ClassVar[int]
    NEXT_HOP_FIELD_NUMBER: _ClassVar[int]
    description: str
    destination: DestinationMatch
    next_hop: NextHop
    def __init__(self, description: _Optional[str] = ..., destination: _Optional[_Union[DestinationMatch, _Mapping]] = ..., next_hop: _Optional[_Union[NextHop, _Mapping]] = ...) -> None: ...

class DestinationMatch(_message.Message):
    __slots__ = ["cidr"]
    CIDR_FIELD_NUMBER: _ClassVar[int]
    cidr: str
    def __init__(self, cidr: _Optional[str] = ...) -> None: ...

class NextHop(_message.Message):
    __slots__ = ["allocation", "default_egress_gateway"]
    ALLOCATION_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_EGRESS_GATEWAY_FIELD_NUMBER: _ClassVar[int]
    allocation: AllocationNextHop
    default_egress_gateway: bool
    def __init__(self, allocation: _Optional[_Union[AllocationNextHop, _Mapping]] = ..., default_egress_gateway: bool = ...) -> None: ...

class AllocationNextHop(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class RouteStatus(_message.Message):
    __slots__ = ["state", "next_hop"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[RouteStatus.State]
        READY: _ClassVar[RouteStatus.State]
    STATE_UNSPECIFIED: RouteStatus.State
    READY: RouteStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    NEXT_HOP_FIELD_NUMBER: _ClassVar[int]
    state: RouteStatus.State
    next_hop: NextHopState
    def __init__(self, state: _Optional[_Union[RouteStatus.State, str]] = ..., next_hop: _Optional[_Union[NextHopState, _Mapping]] = ...) -> None: ...

class NextHopState(_message.Message):
    __slots__ = ["allocation", "default_egress_gateway"]
    ALLOCATION_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_EGRESS_GATEWAY_FIELD_NUMBER: _ClassVar[int]
    allocation: AllocationNextHopState
    default_egress_gateway: DefaultEgressGatewayState
    def __init__(self, allocation: _Optional[_Union[AllocationNextHopState, _Mapping]] = ..., default_egress_gateway: _Optional[_Union[DefaultEgressGatewayState, _Mapping]] = ...) -> None: ...

class AllocationNextHopState(_message.Message):
    __slots__ = ["cidr"]
    CIDR_FIELD_NUMBER: _ClassVar[int]
    cidr: str
    def __init__(self, cidr: _Optional[str] = ...) -> None: ...

class DefaultEgressGatewayState(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.dns.v1``"""

import builtins as builtins
import collections.abc as abc
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import nebius.aio.client as client
import nebius.aio.request as request
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.dns.v1.record_pb2 as record_pb2
import nebius.api.nebius.dns.v1.record_service_pb2 as record_service_pb2
import nebius.api.nebius.dns.v1.zone_pb2 as zone_pb2
import nebius.api.nebius.dns.v1.zone_service_pb2 as zone_service_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import typing as typing
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/dns/v1/record.proto
class Record(pb_classes.Message):
    """
    API Resource: DNS *Resource Record* (RR), an information entry about a specific domain
    
    Each record is contained within a DNS zone, which is a container for DNS data of a specific domain, and, possibly, its subdomains
    DNS zones are represented in this API by the ``Zone`` API Resource which is managed by the ``ZoneService``
    """
    
    __PB2_CLASS__ = record_pb2.Record
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.Record",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "RecordSpec|record_pb2.RecordSpec|None|unset.UnsetType" = unset.Unset,
        status: "RecordStatus|record_pb2.RecordStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        DNS record metadata
        ``metadata.parent_id`` must be a DNS zone ID
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "RecordSpec":
        """
        DNS record specification, including its relative name, type, data and TTL
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=RecordSpec,
        )
    @spec.setter
    def spec(self, value: "RecordSpec|record_pb2.RecordSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "RecordStatus":
        """
        DNS record status, including e.g. its effective FQDN (fully-qualified domain name)
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=RecordStatus,
        )
    @status.setter
    def status(self, value: "RecordStatus|record_pb2.RecordStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class RecordSpec(pb_classes.Message):
    """
    DNS record specification
    """
    
    __PB2_CLASS__ = record_pb2.RecordSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.RecordSpec",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class RecordType(pb_enum.Enum):
        """
        DNS Record type
        """
        
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.dns.v1.RecordSpec.RecordType",record_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        RECORD_TYPE_UNSPECIFIED = 0
        """
        Record type is not specified
        """
        
        A = 1
        """
        ``A`` record: IPv4 address
        """
        
        AAAA = 2
        """
        ``AAAA`` record: IPv6 address
        """
        
        PTR = 3
        """
        ``PTR`` record: mapping from IP address to a domain name
        """
        
        CNAME = 4
        """
        ``CNAME`` record: an alias for a *canonical domain name*
        """
        
        MX = 5
        """
        ``MX`` record: mail server information (domain name, priority)
        """
        
        TXT = 6
        """
        ``TXT`` record: text data, typically used to verify e-mail addresses, websites and TLS certificates
        """
        
        SRV = 7
        """
        ``SRV`` record: information about a network service (domain name, port, weight)
        """
        
        NS = 8
        """
        ``NS`` record: domain name of an authoritative nameserver for this DNS zone, or one of its subzones
        """
        
        SOA = 9
        """
        ``SOA`` record: administrative information about this DNS zone
        """
        
        CAA = 10
        """
        ``CAA`` record: certificate issuance settings for this DNS zone and its subzones
        """
        
        SVCB = 11
        """
        ``SVCB`` record: service binding. See `RFC 9460, section 2.3 <https://www.rfc-editor.org/rfc/rfc9460.html#section-2.3>`_
        """
        
        HTTPS = 12
        """
        ``HTTPS`` record: service binding with HTTPS protocol configuration. See `RFC 9460, section 9.1 <https://www.rfc-editor.org/rfc/rfc9460.html#section-9.1>`_
        """
        
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        relative_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        type: "RecordSpec.RecordType|record_pb2.RecordSpec.RecordType|None|unset.UnsetType" = unset.Unset,
        ttl: "builtins.int|None|unset.UnsetType" = unset.Unset,
        data: "builtins.str|None|unset.UnsetType" = unset.Unset,
        deletion_protection: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(relative_name, unset.UnsetType):
            self.relative_name = relative_name
        if not isinstance(type, unset.UnsetType):
            self.type = type
        if not isinstance(ttl, unset.UnsetType):
            self.ttl = ttl
        if not isinstance(data, unset.UnsetType):
            self.data = data
        if not isinstance(deletion_protection, unset.UnsetType):
            self.deletion_protection = deletion_protection
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "relative_name",
            "type",
            "ttl",
            "data",
            "deletion_protection",
            "RecordType",
        ]
    
    @builtins.property
    def relative_name(self) -> "builtins.str":
        """
        Zone-relative name of this record (e.g., ``www`` for ``www.<parent zone's domain name>``\\ )
        Use ``@`` for records in zone apex (that is, records that have the same domain name as the zone itself)
        To see the resolved absolute domain name, see ``Record.status.effective_fqdn``
        """
        
        return super()._get_field("relative_name", explicit_presence=False,
        )
    @relative_name.setter
    def relative_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("relative_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def type(self) -> "RecordSpec.RecordType":
        """
        Record type
        """
        
        return super()._get_field("type", explicit_presence=False,
        wrap=RecordSpec.RecordType,
        )
    @type.setter
    def type(self, value: "RecordSpec.RecordType|record_pb2.RecordSpec.RecordType|None") -> None:
        return super()._set_field("type",value,explicit_presence=False,
        )
    
    @builtins.property
    def ttl(self) -> "builtins.int":
        """
        Record TTL. If absent or negative, will be assumed to be the default value (\\ ``600``\\ )
        """
        
        return super()._get_field("ttl", explicit_presence=False,
        )
    @ttl.setter
    def ttl(self, value: "builtins.int|None") -> None:
        return super()._set_field("ttl",value,explicit_presence=False,
        )
    
    @builtins.property
    def data(self) -> "builtins.str":
        """
        Record data in text format
        
        This should be the RDATA part of this Resource Record's `presentation (zonefile) format <https://datatracker.ietf.org/doc/html/rfc9499#name-resource-records>`_. E.g., ``10 xyz.tuv`` for a ``@ 600 IN MX 10 xyz.tuv.`` resource record in a zonefile
        """
        
        return super()._get_field("data", explicit_presence=False,
        )
    @data.setter
    def data(self, value: "builtins.str|None") -> None:
        return super()._set_field("data",value,explicit_presence=False,
        )
    
    @builtins.property
    def deletion_protection(self) -> "builtins.bool":
        """
        Mark this record as delete-protected
        Delete-protected records can *only* be deleted by explicitly calling ``RecordService/Delete`` API with ``force`` flag set to ``true``
        """
        
        return super()._get_field("deletion_protection", explicit_presence=False,
        )
    @deletion_protection.setter
    def deletion_protection(self, value: "builtins.bool|None") -> None:
        return super()._set_field("deletion_protection",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "relative_name":"relative_name",
        "type":"type",
        "ttl":"ttl",
        "data":"data",
        "deletion_protection":"deletion_protection",
        "RecordType":"RecordType",
    }
    
class RecordStatus(pb_classes.Message):
    """
    DNS record status
    """
    
    __PB2_CLASS__ = record_pb2.RecordStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.RecordStatus",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        zone_domain_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        effective_fqdn: "builtins.str|None|unset.UnsetType" = unset.Unset,
        reconciling: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(zone_domain_name, unset.UnsetType):
            self.zone_domain_name = zone_domain_name
        if not isinstance(effective_fqdn, unset.UnsetType):
            self.effective_fqdn = effective_fqdn
        if not isinstance(reconciling, unset.UnsetType):
            self.reconciling = reconciling
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "zone_domain_name",
            "effective_fqdn",
            "reconciling",
        ]
    
    @builtins.property
    def zone_domain_name(self) -> "builtins.str":
        """
        Domain name of this record's parent zone including ``.`` at the end, e.g. ``example.com.``
        """
        
        return super()._get_field("zone_domain_name", explicit_presence=False,
        )
    @zone_domain_name.setter
    def zone_domain_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("zone_domain_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def effective_fqdn(self) -> "builtins.str":
        """
        Fully-qualified domain name of this record including ``.`` at the end, e.g. ``www.example.com.``
        """
        
        return super()._get_field("effective_fqdn", explicit_presence=False,
        )
    @effective_fqdn.setter
    def effective_fqdn(self, value: "builtins.str|None") -> None:
        return super()._set_field("effective_fqdn",value,explicit_presence=False,
        )
    
    @builtins.property
    def reconciling(self) -> "builtins.bool":
        """
        Indicates whether there is a running Operation for this Record
        """
        
        return super()._get_field("reconciling", explicit_presence=False,
        )
    @reconciling.setter
    def reconciling(self, value: "builtins.bool|None") -> None:
        return super()._set_field("reconciling",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "zone_domain_name":"zone_domain_name",
        "effective_fqdn":"effective_fqdn",
        "reconciling":"reconciling",
    }
    
class GetRecordRequest(pb_classes.Message):
    """
    Request to get the DNS record by its ID and, optionally, its resource version
    """
    
    __PB2_CLASS__ = record_pb2.GetRecordRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.GetRecordRequest",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        resource_version: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(resource_version, unset.UnsetType):
            self.resource_version = resource_version
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "resource_version",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Record ID
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def resource_version(self) -> "builtins.int":
        """
        Optional: expected record version (\\ ``metadata.resource_version``\\ )
        
        
        * If specified, the requested version will be returned if possible (if the version has changed, you will get a ``FAILED_PRECONDITION`` error)
        * If not specified or set to ``0``\\ , the latest version will be returned
        """
        
        return super()._get_field("resource_version", explicit_presence=False,
        )
    @resource_version.setter
    def resource_version(self, value: "builtins.int|None") -> None:
        return super()._set_field("resource_version",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "resource_version":"resource_version",
    }
    
class ListRecordsRequest(pb_classes.Message):
    """
    Request to list DNS records in the specified parent DNS zone
    """
    
    __PB2_CLASS__ = record_pb2.ListRecordsRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ListRecordsRequest",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Parent DNS zone ID
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        Page size
        If not specified or set to ``0``\\ , a default page size of ``100`` will be used
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        An opaque listing continuation token
        
        
        * If not specified or empty, the first page of results will be returned
        * If specified, the next page of results (as identified by the token) will be returned
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
    }
    
class ListRecordsResponse(pb_classes.Message):
    """
    Response with a single page of DNS record listing results
    """
    
    __PB2_CLASS__ = record_pb2.ListRecordsResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ListRecordsResponse",record_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[Record]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Record]":
        """
        This page's items
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Record,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Record]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        An opaque listing continuation token that can be used to return the next page of listing results
        If ``next_page_token`` is empty, this is the last result page
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    
# file: nebius/dns/v1/record_service.proto

class RecordServiceClient(client.Client):
    """
    Service for managing DNS records (\\ ``Record`` resources)
    
    This class provides the client methods for the ``.nebius.dns.v1.RecordService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.dns.v1.RecordService",record_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.dns.v1.RecordService"
    
    def get(self,
        request: "GetRecordRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["GetRecordRequest","Record"]:
        """
        Gets the DNS record with the specified ID
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.dns.v1.GetRecordRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.Record`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=record_pb2.Record,
            result_wrapper=pb_classes.simple_wrapper(Record),
            **kwargs,
        )
    
    def get_by_name(self,
        request: "v1_1.GetByNameRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["v1_1.GetByNameRequest","Record"]:
        """
        Gets the DNS record by its parent DNS zone (\\ ``metadata.parent_id``\\ ) and ``metadata.name``
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.common.v1.GetByNameRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.Record`.
        """
        
        return super().request(
            method="GetByName",
            request=request,
            result_pb2_class=record_pb2.Record,
            result_wrapper=pb_classes.simple_wrapper(Record),
            **kwargs,
        )
    
    def list(self,
        request: "ListRecordsRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["ListRecordsRequest","ListRecordsResponse"]:
        """
        Lists DNS records in the specified DNS zone
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.dns.v1.ListRecordsRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.ListRecordsResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=record_pb2.ListRecordsResponse,
            result_wrapper=pb_classes.simple_wrapper(ListRecordsResponse),
            **kwargs,
        )
    

# file: nebius/dns/v1/zone.proto
class Zone(pb_classes.Message):
    """
    API Resource: *DNS zone*\\ , a container for DNS data
    
    Each DNS zone starts at a particular domain within the hierarchical DNS namespace tree, e.g., ``example.com.``\\ , and can also include its subdomains, e.g., ``sales.example.com.``
    DNS Zones contain *Resource Records* (RRs), which are individual information entries about the domain(s), e.g., a domain's IP address
    See the `graphical explanation of DNS zones and Resource Records on Wikipedia <https://en.wikipedia.org/wiki/Domain_Name_System#/media/File:Domain_name_space.svg>`_
    
    A Resource Record is represented in this API by the ``Record`` API Resource which is managed by the ``RecordService``
    """
    
    __PB2_CLASS__ = zone_pb2.Zone
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.Zone",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "ZoneSpec|zone_pb2.ZoneSpec|None|unset.UnsetType" = unset.Unset,
        status: "ZoneStatus|zone_pb2.ZoneStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        DNS zone metadata
        ``metadata.parent_id`` must be an IAM Container ID
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "ZoneSpec":
        """
        DNS zone specification, including its domain name and scope
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=ZoneSpec,
        )
    @spec.setter
    def spec(self, value: "ZoneSpec|zone_pb2.ZoneSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "ZoneStatus":
        """
        DNS zone status, including e.g. the number of records in the zone
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=ZoneStatus,
        )
    @status.setter
    def status(self, value: "ZoneStatus|zone_pb2.ZoneStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class ZoneSpec(pb_classes.Message):
    """
    DNS Zone specification
    """
    
    __PB2_CLASS__ = zone_pb2.ZoneSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ZoneSpec",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass_scope__(pb_classes.OneOf):
        name: builtins.str= "scope"
        
        def __init__(self, msg: "ZoneSpec") -> None:
            super().__init__()
            self._message: "ZoneSpec" = msg
    
    class __OneOfClass_scope_vpc__(__OneOfClass_scope__):
        field: typing.Literal["vpc"] = "vpc"
        
        def __init__(self, msg: "ZoneSpec") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "VpcZoneScope":
            return self._message.vpc
    
    @builtins.property
    def scope(self) -> __OneOfClass_scope_vpc__|None:
        """
        Scope for this zone
        The scope is chosen at zone creation time and cannot be changed, but scope's attributes can sometimes be changed
        """
        
        field_name_1: str|None = super().which_field_in_oneof("scope")
        match field_name_1:
            case "vpc":
                return self.__OneOfClass_scope_vpc__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        domain_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        vpc: "VpcZoneScope|zone_pb2.VpcZoneScope|None|unset.UnsetType" = unset.Unset,
        soa_spec: "SoaSpec|zone_pb2.SoaSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(domain_name, unset.UnsetType):
            self.domain_name = domain_name
        if not isinstance(vpc, unset.UnsetType):
            self.vpc = vpc
        if not isinstance(soa_spec, unset.UnsetType):
            self.soa_spec = soa_spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "domain_name",
            "vpc",
            "soa_spec",
            "scope",
        ]
    
    @builtins.property
    def domain_name(self) -> "builtins.str":
        """
        Fully qualified domain name of this zone, including ``.`` at the end
        """
        
        return super()._get_field("domain_name", explicit_presence=False,
        )
    @domain_name.setter
    def domain_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("domain_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def vpc(self) -> "VpcZoneScope|None":
        """
        A VPC zone, which is only visible from a virtual network (VPC)
        """
        
        return super()._get_field("vpc", explicit_presence=True,
        wrap=VpcZoneScope,
        )
    @vpc.setter
    def vpc(self, value: "VpcZoneScope|zone_pb2.VpcZoneScope|None") -> None:
        return super()._set_field("vpc",value,explicit_presence=True,
        )
    
    @builtins.property
    def soa_spec(self) -> "SoaSpec":
        """
        Custom SOA (Start of Authority) record specification for the zone
        """
        
        return super()._get_field("soa_spec", explicit_presence=False,
        wrap=SoaSpec,
        )
    @soa_spec.setter
    def soa_spec(self, value: "SoaSpec|zone_pb2.SoaSpec|None") -> None:
        return super()._set_field("soa_spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "domain_name":"domain_name",
        "vpc":"vpc",
        "soa_spec":"soa_spec",
        "scope":"scope",
    }
    
class VpcZoneScope(pb_classes.Message):
    """
    VPC zone scope specification
    """
    
    __PB2_CLASS__ = zone_pb2.VpcZoneScope
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.VpcZoneScope",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        primary_network_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(primary_network_id, unset.UnsetType):
            self.primary_network_id = primary_network_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "primary_network_id",
        ]
    
    @builtins.property
    def primary_network_id(self) -> "builtins.str":
        """
        ID of the virtual network that this zone's records will be visible from
        This value cannot be changed after creating the zone
        """
        
        return super()._get_field("primary_network_id", explicit_presence=False,
        )
    @primary_network_id.setter
    def primary_network_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("primary_network_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "primary_network_id":"primary_network_id",
    }
    
class SoaSpec(pb_classes.Message):
    """
    Custom SOA (Start of Authority) record specification for the zone
    """
    
    __PB2_CLASS__ = zone_pb2.SoaSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.SoaSpec",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        negative_ttl: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(negative_ttl, unset.UnsetType):
            self.negative_ttl = negative_ttl
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "negative_ttl",
        ]
    
    @builtins.property
    def negative_ttl(self) -> "builtins.int":
        """
        Specifies TTL, in seconds, for caching ``NXDOMAIN`` ("record not found") DNS responses from this zone (\\ *negative caching*\\ )
        Set this TTL to a low value if you frequently delete and recreate records instead of updating them
        *Note:* Values of less than ``5`` will be ignored, and a default negative caching TTL will be used instead
        """
        
        return super()._get_field("negative_ttl", explicit_presence=False,
        )
    @negative_ttl.setter
    def negative_ttl(self, value: "builtins.int|None") -> None:
        return super()._set_field("negative_ttl",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "negative_ttl":"negative_ttl",
    }
    
class ZoneStatus(pb_classes.Message):
    """
    DNS zone status
    """
    
    __PB2_CLASS__ = zone_pb2.ZoneStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ZoneStatus",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        record_count: "builtins.int|None|unset.UnsetType" = unset.Unset,
        reconciling: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(record_count, unset.UnsetType):
            self.record_count = record_count
        if not isinstance(reconciling, unset.UnsetType):
            self.reconciling = reconciling
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "record_count",
            "reconciling",
        ]
    
    @builtins.property
    def record_count(self) -> "builtins.int":
        """
        Number of records in this zone. May be 0 if not calculated (e.g., in listings)
        """
        
        return super()._get_field("record_count", explicit_presence=False,
        )
    @record_count.setter
    def record_count(self, value: "builtins.int|None") -> None:
        return super()._set_field("record_count",value,explicit_presence=False,
        )
    
    @builtins.property
    def reconciling(self) -> "builtins.bool":
        """
        Indicates whether there is a running Operation for this Zone
        """
        
        return super()._get_field("reconciling", explicit_presence=False,
        )
    @reconciling.setter
    def reconciling(self, value: "builtins.bool|None") -> None:
        return super()._set_field("reconciling",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "record_count":"record_count",
        "reconciling":"reconciling",
    }
    
class GetZoneRequest(pb_classes.Message):
    """
    Request to get the DNS zone by its ID and, optionally, its resource version
    """
    
    __PB2_CLASS__ = zone_pb2.GetZoneRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.GetZoneRequest",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        resource_version: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(resource_version, unset.UnsetType):
            self.resource_version = resource_version
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "resource_version",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Zone ID
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def resource_version(self) -> "builtins.int":
        """
        Optional: expected zone version (\\ ``metadata.resource_version``\\ )
        
        
        * If specified, the requested version will be returned if possible (if the version has changed, you will get a ``FAILED_PRECONDITION`` error)
        * If not specified or set to ``0``\\ , the latest zone version will be returned
        """
        
        return super()._get_field("resource_version", explicit_presence=False,
        )
    @resource_version.setter
    def resource_version(self, value: "builtins.int|None") -> None:
        return super()._set_field("resource_version",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "resource_version":"resource_version",
    }
    
class ListZonesRequest(pb_classes.Message):
    """
    Request to list DNS zones in the specified parent IAM container
    """
    
    __PB2_CLASS__ = zone_pb2.ListZonesRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ListZonesRequest",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Parent IAM container ID
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        Page size
        If not specified or set to ``0``\\ , a default page size of ``100`` will be used
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        An opaque listing continuation token
        
        
        * If not specified or empty, the first page of results will be returned
        * If specified, the next page of results (as identified by the token) will be returned
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
    }
    
class ListZonesResponse(pb_classes.Message):
    """
    Response with a single page of DNS zone listing results
    """
    
    __PB2_CLASS__ = zone_pb2.ListZonesResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.dns.v1.ListZonesResponse",zone_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[Zone]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Zone]":
        """
        This page's items
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Zone,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Zone]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        An opaque listing continuation token that can be used to return the next page of listing results
        If ``next_page_token`` is empty, this is the last result page
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    
# file: nebius/dns/v1/zone_service.proto

class ZoneServiceClient(client.Client):
    """
    Service for managing DNS zones (\\ ``Zone`` resources)
    
    This class provides the client methods for the ``.nebius.dns.v1.ZoneService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.dns.v1.ZoneService",zone_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.dns.v1.ZoneService"
    
    def get(self,
        request: "GetZoneRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["GetZoneRequest","Zone"]:
        """
        Gets the DNS zone with the specified ID
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.dns.v1.GetZoneRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.Zone`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=zone_pb2.Zone,
            result_wrapper=pb_classes.simple_wrapper(Zone),
            **kwargs,
        )
    
    def get_by_name(self,
        request: "v1_1.GetByNameRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["v1_1.GetByNameRequest","Zone"]:
        """
        Gets the DNS zone by its parent IAM container (\\ ``metadata.parent_id``\\ ) and ``metadata.name``
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.common.v1.GetByNameRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.Zone`.
        """
        
        return super().request(
            method="GetByName",
            request=request,
            result_pb2_class=zone_pb2.Zone,
            result_wrapper=pb_classes.simple_wrapper(Zone),
            **kwargs,
        )
    
    def list(self,
        request: "ListZonesRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["ListZonesRequest","ListZonesResponse"]:
        """
        Lists DNS zones in the specified parent IAM container
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.dns.v1.ListZonesRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.dns.v1.ListZonesResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=zone_pb2.ListZonesResponse,
            result_wrapper=pb_classes.simple_wrapper(ListZonesResponse),
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "Record",
    "RecordSpec",
    "RecordStatus",
    "GetRecordRequest",
    "ListRecordsRequest",
    "ListRecordsResponse",
    "RecordServiceClient",
    "Zone",
    "ZoneSpec",
    "VpcZoneScope",
    "SoaSpec",
    "ZoneStatus",
    "GetZoneRequest",
    "ListZonesRequest",
    "ListZonesResponse",
    "ZoneServiceClient",
]



"""Idempotency key interceptor for gRPC aio clients.

This module provides functionality to automatically add idempotency keys to
gRPC client calls, ensuring that operations can be safely retried without
causing duplicate side effects on the server.

The interceptor adds a unique UUID4 key to the 'x-idempotency-key' header
for each call that doesn't already have one.
"""

from collections.abc import Callable
from logging import getLogger
from typing import TypeVar
from uuid import uuid4

from grpc.aio._call import UnaryUnaryCall
from grpc.aio._interceptor import ClientCallDetails, UnaryUnaryClientInterceptor
from grpc.aio._metadata import Metadata as GRPCMetadata

from nebius.base.metadata import Metadata

log = getLogger(__name__)

HEADER = "x-idempotency-key"
"""The gRPC metadata header name used for idempotency keys."""

Req = TypeVar("Req")
Res = TypeVar("Res")


def new_key() -> str:
    """Generate a new idempotency key.

    :returns: A new UUID4 string to use as an idempotency key.
    """
    return str(uuid4())


def add_key_to_metadata(metadata: Metadata | GRPCMetadata) -> None:
    """Add a new idempotency key to the provided metadata.

    :param metadata: The metadata object to add the key to.
    """
    log.debug("added idempotency key to metadata")
    metadata[HEADER] = new_key()


def ensure_key_in_metadata(metadata: Metadata | GRPCMetadata) -> None:
    """Ensure an idempotency key is present in the metadata.

    If no idempotency key exists or it's empty, a new one is added.

    :param metadata: The metadata object to check and potentially modify.
    """
    if HEADER not in metadata or metadata[HEADER] == "" or metadata[HEADER] == [""]:
        add_key_to_metadata(metadata)


class IdempotencyKeyInterceptor(UnaryUnaryClientInterceptor):  # type: ignore[unused-ignore,misc]
    """gRPC client interceptor that adds idempotency keys to unary-unary calls.

    This interceptor ensures that every gRPC call has an idempotency key in its
    metadata, which helps prevent duplicate operations on the server side.
    """

    async def intercept_unary_unary(
        self,
        continuation: Callable[[ClientCallDetails, Req], UnaryUnaryCall | Res],
        client_call_details: ClientCallDetails,
        request: Req,
    ) -> UnaryUnaryCall | Res:
        """Intercept a unary-unary gRPC call and ensure idempotency key is present.

        :param continuation: The next interceptor in the chain or the actual call.
        :param client_call_details: Details of the client call, including metadata.
        :param request: The request payload.
        :returns: The result of the gRPC call.
        """
        if client_call_details.metadata is None:
            client_call_details.metadata = GRPCMetadata()
        ensure_key_in_metadata(client_call_details.metadata)
        return await continuation(client_call_details, request)  # type: ignore



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.mysterybox.v1 import secret_pb2 as nebius_dot_mysterybox_dot_v1_dot_secret__pb2
from nebius.api.nebius.mysterybox.v1 import secret_service_pb2 as nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2


class SecretServiceStub(object):
    """A set of methods for managing secrets.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/Create',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.CreateSecretRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Update = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/Update',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UpdateSecretRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Get = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/Get',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretRequest.SerializeToString,
                response_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.FromString,
                )
        self.GetByName = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/GetByName',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretByNameRequest.SerializeToString,
                response_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.FromString,
                )
        self.List = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/List',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsRequest.SerializeToString,
                response_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsResponse.FromString,
                )
        self.Delete = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/Delete',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.DeleteSecretRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Undelete = channel.unary_unary(
                '/nebius.mysterybox.v1.SecretService/Undelete',
                request_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UndeleteSecretRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )


class SecretServiceServicer(object):
    """A set of methods for managing secrets.
    """

    def Create(self, request, context):
        """Creates a new secret in the specified container.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Updates an existing secret, excluded its Payload.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Returns the specified secret, without its Payload.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetByName(self, request, context):
        """Returns the specified secret by name, without its Payload.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Returns the list of secrets in a specified container, without its Payload.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Soft delete secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Undelete(self, request, context):
        """Undelete secret version
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SecretServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.CreateSecretRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UpdateSecretRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretRequest.FromString,
                    response_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.SerializeToString,
            ),
            'GetByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetByName,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretByNameRequest.FromString,
                    response_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsRequest.FromString,
                    response_serializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsResponse.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.DeleteSecretRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Undelete': grpc.unary_unary_rpc_method_handler(
                    servicer.Undelete,
                    request_deserializer=nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UndeleteSecretRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.mysterybox.v1.SecretService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SecretService(object):
    """A set of methods for managing secrets.
    """

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/Create',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.CreateSecretRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/Update',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UpdateSecretRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/Get',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretRequest.SerializeToString,
            nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/GetByName',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.GetSecretByNameRequest.SerializeToString,
            nebius_dot_mysterybox_dot_v1_dot_secret__pb2.Secret.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/List',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsRequest.SerializeToString,
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.ListSecretsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/Delete',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.DeleteSecretRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Undelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.mysterybox.v1.SecretService/Undelete',
            nebius_dot_mysterybox_dot_v1_dot_secret__service__pb2.UndeleteSecretRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
